; ### SFX edit ###
; For 24k.

; $1000 - $11ff : vmatrix
; $1201 - $12xx : basic start
; $12xx - $1fff : ml entry, main code, RO data
; $2e00 - $2fff : mul 5 LUT
; $3000 - $7fff : sound data

!ct "text_convtab.bin"

; ###############

; # const

DEBUG_IRQ_LEN = 0

COLS = 17
ROWS = 23

SOUND_VOLUME = 8

COLOR_BLACK = 0
COLOR_WHITE = 1
COLOR_RED = 2
COLOR_CYAN = 3
COLOR_PURPLE = 4
COLOR_GREEN = 5
COLOR_BLUE = 6
COLOR_YELLOW = 7

NORMAL_900F = (COLOR_WHITE << 4) | 8 | COLOR_CYAN

LEN_SFXITEM = 5
LEN_SFXDATA = LEN_SFXITEM * $100
LEN_ALLSFX = LEN_SFXDATA * $10

SFXBYTE_NOP = $7f
SFXBYTE_STOP = $7e

; ###############

; # addr

irq_downcount = $39
zptmp = $3a
zpptr = $3b
zpptrh = $3c
sfxptr = $3d
sfxptrh = $3e
sound_enabled = $3f
sound_pos = $40
sound_num = $41

pcur_pos = $42
temp_line = $45
temp_len = $46
temp_key = $47

edit_pos = $48  ; data line position, $00 .. $ff
edit_offs = $49 ; data window offset, $00 .. $ff
edit_ch = $4a   ; channel to edit, 0 .. 4
edit_curxp = $61    ; edit cursor x, 2 + edit_ch * 3
edit_curyp = $62    ; edit cursor y, edit_pos - edit_offs + 1

k_status_word = $90
k_device_num = $ba

; $c5   ; used by keyscan
; $c6   ; used by keyscan
; $cb   ; used by keyscan
; $f5   ; used by keyscan
; $f6   ; used by keyscan

; used by keyscan etc
; $277
; $278
kv_hibase = $288
; $289
kv_rptflg = $28a
; $28c
; $28d
; $28e
; $28f
; $290
kv_mode = $291

vmatrix = $1000
charset = $8800
colorram = $9400 + (vmatrix & $200)

; item = index * 5
tbl_mul5 = $2e00
tbl_mul5h = $2f00

; raw sound data:
;  CA CB CC CD CE  * $100 lines  * $10 sounds
;  either
;   value to write to $900a .. $900e
;  or
;   SFXBYTE_NOP
;   SFXBYTE_STOP
sounddata = $3000

k_getinkbd = $f1f9 ;$ffe4
k_unlisten = $ffae
k_listen = $ffb1
k_setlfs = $ffba
k_setnam = $ffbd
k_open = $ffc0
k_close = $ffc3
k_chkin = $ffc6
k_chrin = $ffcf
k_chrout = $ffd2
k_load = $ffd5
k_save = $ffd8
k_getin = $ffe4

 
; ###############

!zn Basic_entry {
* = $1201
basic_start:
!wo basic_stop
!wo 0
!pet $9e, "4621", 0
basic_stop:
!wo 0
ml_entry:
    jmp init_phase0
} ; !zn Basic_entry

; ###############

!zn INT_Handler {
irq_handler:
    lda $9124   ; ACK IRQ
!if DEBUG_IRQ_LEN > 0 {
    lda #NORMAL_900F XOR $11
    sta $900f
}
    jsr sound_tick

    ldy irq_downcount
    beq +
    dey
    sty irq_downcount
+
    ; read the keyboard
    jsr $ff9f

irq_exit:
; DEBUG
!if DEBUG_IRQ_LEN > 0 {
    lda #NORMAL_900F
    sta $900f
}

;brk_handler:
    pla
    tay
    pla
    tax
    pla

nmi_handler:
    rti

brk_handler:
!by 2   ; JAM

}   ; !zn INT_Handler


; ###############

; # init code

!zn Phase0_Init {

.SCREEN_X_P = 12
.SCREEN_Y_P = 38
.SCREEN_X_N = 5
.SCREEN_Y_N = 25
.RASTER_P = 131
.RASTER_N = 118

; # p0 data

.gamefw_tv_tbl_vic:
!by .SCREEN_X_P, .SCREEN_Y_P, (((vmatrix & $200) >> 2) | COLS), ROWS << 1, 0
!by ((((vmatrix & $1c00) >> 10) | ((vmatrix & $8000) >> 12) XOR 8) << 4) | (((charset & $1c00) >> 10) | ((charset & $8000) >> 12) XOR 8)
!by 0, 0, 0, 0
!by 0, 0, 0, 0
!by SOUND_VOLUME, NORMAL_900F

; tbl: index 0 -> pal, 1 -> ntsc
.gamefw_tv_tbl_raster:
!by .RASTER_P, .RASTER_N
.gamefw_tv_tbl_timer_l:
!by <(312 * 71 - 2), <(261 * 65 - 2)
.gamefw_tv_tbl_timer_h:
!by >(312 * 71 - 2), >(261 * 65 - 2)

.text_starttext:
;    01234567890123456
!tx "F1 to keep data  "
!tx "in RAM or any    "
!tx "other to clear.  "
LEN_text_starttext = * - .text_starttext

; # p0 code

init_phase0:
    sei
    ldx #$e0    ; leave room to top of page 1
    txs
    cld
    lda #$7f
    sta $913d
    sta $913e

    ; initialize VIC/VIA
    ldx #$f
-   lda .gamefw_tv_tbl_vic,x
    sta $9000,x
    dex
    bpl -
    lda #0
    ldx #$a
-   sta $9132,x
    dex
    bpl -

    ; setup screen
    ldx #0
-   lda #' '
    sta vmatrix,x
    sta vmatrix + COLS * ROWS - $100,x
    lda #COLOR_BLUE
    sta colorram,x
    sta colorram + COLS * ROWS - $100,x
    inx
    bne -

    ldx #LEN_text_starttext
-   lda .text_starttext - 1,x
    sta vmatrix - 1,x
    dex
    bne -

    ; make LUTs
    ldx #0
    txa
    tay
-   sta tbl_mul5,x
    pha
    tya
    sta tbl_mul5h,x
    pla
    clc
    adc #5
    bcc +
    iny
+   inx
    bne -

    ; init variables
    stx sound_enabled
    stx sound_num
    stx pcur_pos
    stx edit_pos
    stx edit_offs
    stx edit_ch
    stx edit_curyp

    lda #2
    sta edit_curxp
    lda #<sounddata
    sta sfxptr
    lda #>sounddata
    sta sfxptrh

    ; detect pal/ntsc
    lda $ede4
    and #$01
    tax ; 0 -> pal, 1 -> ntsc
    beq +
    lda #.SCREEN_X_N
    sta $9000
    lda #.SCREEN_Y_N
    sta $9001
+
    lda #>vmatrix
    sta kv_hibase
    lda #$80
    sta kv_rptflg
    sta kv_mode

    ; enable Timer A free run on VIA 1
    lda #$40
    sta $912b

    lda #<irq_handler
    sta $0314
    lda #>irq_handler
    sta $0315
    lda #<brk_handler
    sta $0316
    lda #>brk_handler
    sta $0317
    lda #<nmi_handler
    sta $0318
    lda #>nmi_handler
    sta $0319

    lda .gamefw_tv_tbl_raster,x
    pha
    lda .gamefw_tv_tbl_timer_l,x
    tay
    lda .gamefw_tv_tbl_timer_h,x
    tax
    ; synchronise with the screen
    pla
-   cmp $9004
    beq -
-   cmp $9004
    bne -

    sty $9126
    stx $9125

    lda #$c0
    cli
    jmp main_entry
} ; !zn Phase0_Ini


; ###############

; # (mostly) RO data

!zn Main_data {

tbl_soundptr:
!for .i, 0, $f {
!by <(sounddata + .i * LEN_SFXDATA)
}
tbl_soundptrh:
!for .i, 0, $f {
!by >(sounddata + .i * LEN_SFXDATA)
}

tbl_screenptr:
!for .i, 0, ROWS - 1 {
!by <(vmatrix + .i * COLS)
}
tbl_screenptrh:
!for .i, 0, ROWS - 1 {
!by >(vmatrix + .i * COLS)
}

hexchars:
!tx "0123456789abcdef"
hexchars_pet:
!pet "0123456789abcdef"

tbl_ch_cur_offs:
!for .i, 0, 5 {
!by 2 + .i * 3
}

filename_s:
!pet "s:"
filename:
!pet "sfxdata"
filenamedigit:
!pet "0"
LEN_filename = * - filename
LEN_filename_s = * - filename_s

text_topline:
;    01234567890123456
!tx "Sx CA CB CC CD CE"
LEN_text_topline = * - text_topline

text_clear:
!tx "Clear", 0
text_load:
!tx "Load", 0
text_save:
!tx "Save", 0
text_qya:
!tx "? (y/a)", 0

} ; !zn Main_data


; ###############

; # utility code

!zn Util_code {

; in:
; a = char
; x = column
; y = line
; out:
; x++
print_char_xy:
    pha
    jsr print_set_line
    pla
    ; FALL THROUGH

; in:
; a = char
; x = column
; out:
; x++
print_char:
print_char_smc_linel = * + 1
print_char_smc_lineh = * + 2
    sta vmatrix,x
    inx
    rts

; y = line
print_set_line:
    lda tbl_screenptr,y
    sta print_char_smc_linel
    lda tbl_screenptrh,y
    sta print_char_smc_lineh
    rts

; in:
; a = item
; x = column
; Z = item == 0
; out:
; x += 3
print_item:
    bne +
    lda #'-'
-   jsr print_char
    jsr print_char
    inx
    rts
+   cmp #SFXBYTE_NOP
    bne +
    lda #'.'
    bne -   ; bra
+   cmp #SFXBYTE_STOP
    bne +
    lda #'S'
    bne -   ; bra
+
    ; FALL THROUGH

; in:
; a = number
; x = column
; out:
; x += 3
print_hex:
    sty .print_hex_y
    pha
    lsr
    lsr
    lsr
    lsr
    tay
    lda hexchars,y
    jsr print_char
    iny
    pla
    and #$f
    tay
    lda hexchars,y
    jsr print_char
    inx
.print_hex_y = * + 1
    ldy #0
    rts


print_clear_statusline:
    ldy #ROWS - 1
    jsr print_set_line
    lda #' '
    ldx #COLS - 1
-   sta vmatrix + (ROWS - 1) * COLS,x
    dex
    bpl -
    rts

; in:
; a:y -> string
; x = column
; out:
; x += len(string)
print_str:
    sta print_str_l
    sty print_str_h
    ldy #0
print_str_l = * + 1
print_str_h = * + 2
-   lda $1000,y
    beq +
    jsr print_char
    iny
    bne -
+   rts

print_data_window:
    ldx edit_offs
    stx zptmp   ; data position
    lda #1
    sta temp_line
--  ldy temp_line
    cpy #ROWS - 1
    bne +
    rts
+   ldx zptmp   ; data position
    jsr set_data_ptr_x
    jsr print_set_line
    dey
    tya
    clc
    adc edit_offs
    ldx #0
    jsr print_hex
    ldy #0
-   lda (zpptr),y
    jsr print_item
    iny
    cpy #5
    bne -
    inc zptmp   ; data position
    inc temp_line
    bne --  ; bra
    rts


; out:
; a = temp_key = key
get_input_draw_pcur:
--  lda #1
    sta irq_downcount
-   lda irq_downcount
    bne -
    ; erase play cursor position
    ldy pcur_pos
    ldx #2
    lda #' '
    jsr print_char_xy
    ; if in edit area,
    ; draw edit cursor (just in case it's equal to play cursor)
    ldy edit_curyp
    beq +
    ldx edit_curxp
    lda #'>'
    jsr print_char_xy
+   ; if playing...
    lda sound_enabled
    beq get_input_draw_pcur_draw_done
    ; calculate new position
    ldy #(ROWS - 1) - 1  ; last - top line
    ; pos B { offs .. offs + ROWS - 2 }
    lda sound_pos
    sec
    sbc edit_offs
    bcs ++
    ; pos < offs
    ldy #-1 ; first - top line
++  ; check if ((pos - offs) & 0xff) < ( ROWS - 1)
    cmp #(ROWS - 1)
    bcc get_input_draw_pcur_draw_got_y
    tya ; cursor is off the edit area
get_input_draw_pcur_draw_got_y:
    tay
    iny ; + top line
    ; store and draw play cursor
    sty pcur_pos
    ldx #2
    lda #'<'
    jsr print_char_xy
get_input_draw_pcur_draw_done:
    ; get key
    jsr k_getinkbd
    beq --
    sta temp_key
    rts


; in:
; a = hex digit (ASCII)
; out:
; C = 0 if input ok
; a = value
hexkey_to_val:
    sec
    sbc #$30
    bcs +
--  sec
    rts
+   cmp #9 + 1
    bcs +
    rts
+   ; sec
    sbc #$41 - $30 - $a
    cmp #$a
    bcc --
    cmp #$f + 1
    rts


; in:
; a = hex digit (ASCII)
; out:
; C = 0 if input ok
; a = value
get_other_hex:
    jsr hexkey_to_val
    bcc +
    rts
+   asl
    asl
    asl
    asl
    sta zptmp
    jsr get_input_draw_pcur
    jsr hexkey_to_val
    bcs +
    ; clc
    adc zptmp
    ; clc
+   rts


; out:
; zpptr:h -> data line
; y = data column
get_data_pos:
    ldx edit_pos
    ldy edit_ch
    ; FALL THROUGH

; in:
; x = position (line)
; out:
; zpptr:h -> data line
set_data_ptr_x:
    lda tbl_mul5,x
    clc
    adc sfxptr
    sta zpptr
    lda tbl_mul5h,x
    adc sfxptrh
    sta zpptrh
    rts

; in:
; zpptr:h -> stop line
; out:
; N = 1
erase_stop_line:
    ldy #4
    lda #SFXBYTE_NOP
-   sta (zpptr),y
    dey
    bpl -
    rts

enter_stop:
    ; search and delete current SFXBYTE_STOP
    ldx #0
    ldy #0
-   jsr set_data_ptr_x
    lda (zpptr),y
    cmp #SFXBYTE_STOP
    bne +
    ; found it, replace with NOPs
    jsr erase_stop_line
    bmi ++  ; bra
+   inx
    bne -
++
    ; enter the new SFXBYTE_STOP
    jsr get_data_pos
    ldy #4
    lda #SFXBYTE_STOP
-   sta (zpptr),y
    dey
    bpl -
    rts

; in:
; a = value
enter_data:
    ; if value is SFXBYTE_STOP ...
    cmp #SFXBYTE_STOP
    beq enter_stop
    sta zptmp   ; value
    ; point to current position in data
    jsr get_data_pos
    ; check if the old value was SFXBYTE_STOP
    lda (zpptr),y
    cmp #SFXBYTE_STOP
    bne +
    ; erase stop line
    jsr erase_stop_line
    ldy edit_ch
+   ; store data to position
    lda zptmp
    sta (zpptr),y
    rts


; in:
; a:y -> text
; out:
; N = 1 if answer invalid
; C = 1 if yes, 0 if all
ask_ya:
    ldx #0
    jsr print_str
    lda #<text_qya
    ldy #>text_qya
    jsr print_str
    jsr get_input_draw_pcur
    pha
    jsr print_clear_statusline
    pla
    cmp #$59    ; y
    bne +
    sec
    rts
+   cmp #$41    ; a
    bne +
    clc
    rts
+   lda #-1
    rts

clear_but_ask:
    lda #<text_clear
    ldy #>text_clear
    jsr ask_ya
    bpl +
    rts
+   bcs +++
    ; all
    jsr sounddata_clear_all
    rts
+++ ; yes
    ldx sound_num
    jsr sounddata_clear_x
    rts

load_but_ask:
    lda #<text_load
    ldy #>text_load
    jsr ask_ya
    bpl +
    rts
+   bcs +++
    ; all
    jsr sounddata_load_all
    rts
+++ ; yes
    ldx sound_num
    jsr sounddata_load_x
    rts

save_but_ask:
    lda #<text_save
    ldy #>text_save
    jsr ask_ya
    bpl +
    rts
+   bcs +++
    ; all
    jsr sounddata_save_all
    rts
+++ ; yes
    ldx sound_num
    jsr sounddata_save_x
    rts

} ; !zn Util_code


; ###############

; # main code

!zn Main_code {

main_entry:
    ; ask if clear data
    jsr get_input_draw_pcur
    cmp #$85    ; f1
    beq +
    jsr sounddata_clear_all
+
    ; erase screen
    ldx #0
    lda #' '
-   sta vmatrix,x
    sta vmatrix + COLS * ROWS - $100,x
    inx
    bne -

    ldx #LEN_text_topline
-   lda text_topline - 1,x
    sta vmatrix - 1,x
    dex
    bne -

    ;ldx #0

main_select:
    jsr sound_select

main_redraw:
    jsr print_data_window

main_loop:
    ; erase edit cursor
    ldx edit_curxp
    ldy edit_curyp
    lda #' '
    jsr print_char_xy
    ; draw edit cursor
    ldy edit_ch
    ldx tbl_ch_cur_offs,y
    stx edit_curxp
    lda edit_pos
    sec
    sbc edit_offs
    tay
    iny
    sty edit_curyp
    lda #'>'
    jsr print_char_xy
    jsr get_input_draw_pcur

    ; print key hex value
    ldy #ROWS - 1
    jsr print_set_line
    ldx #COLS - 2
    lda temp_key
    jsr print_hex

    lda temp_key
    cmp #$03    ; run/stop
    bne ++
    jsr sound_stop
    jmp main_loop
++
    cmp #$20    ; space
    bne ++
    jsr sound_play
    jmp main_loop
++
    cmp #$49    ; i
    bne ++
-   ; up
    dec edit_pos
    lda edit_curyp
    cmp #1
    bne +
    dec edit_offs
    jmp main_redraw
+   jmp main_loop
++
    cmp #$91    ; crsr up
    beq -
    cmp #$4b    ; k
    bne ++
-   ; down
    inc edit_pos
    lda edit_curyp
    cmp #ROWS - 2
    bne +
    inc edit_offs
    jmp main_redraw
+   jmp main_loop
++
    cmp #$11    ; crsr down
    beq -
    cmp #$4a    ; j
    bne ++
-   ; left
    dec edit_ch
    bpl +
    lda #4
    sta edit_ch
+   jmp main_loop
++
    cmp #$9d    ; crsr left
    beq -
    cmp #$4c    ; l
    bne ++
-   ; right
    ldy edit_ch
    iny
    cpy #5
    bne +
    ldy #0
+   sty edit_ch
    jmp main_loop
++
    cmp #$1d    ; crsr right
    beq -
    cmp #$55    ; u
    bne ++
    ; page up
    lda edit_pos
    sec
    sbc #ROWS - 2
    sta edit_pos
    lda edit_offs
    sec
    sbc #ROWS - 2
    sta edit_offs
    jmp main_redraw
++
    cmp #$4f    ; o
    bne ++
    ; page down
    lda edit_pos
    clc
    adc #ROWS - 2
    sta edit_pos
    lda edit_offs
    clc
    adc #ROWS - 2
    sta edit_offs
    jmp main_redraw
++
    cmp #$ce    ; N
    bne ++
    ; prev sfx
    ldx sound_num
    dex
    bpl +
    ldx #$f
+   jmp main_select
++
    cmp #$cd    ; M
    bne ++
    ; next fx
    ldx sound_num
    inx
    cpx #$10
    bne +
    ldx #$0
+   jmp main_select
++
    ; 0..9
    cmp #$30
    bcc ++
    cmp #$39 + 1
    bcs ++
-   jsr get_other_hex
    bcs +
    jsr enter_data
    jmp main_redraw
+   jmp main_loop
++
    ; a..f
    cmp #$41
    bcc ++
    cmp #$46 + 1
    bcc -
++
    cmp #$53    ; s
    bne ++
    ; enter STOP
    jsr enter_stop
    jmp main_redraw
++
    cmp #$2e    ; .
    bne ++
    ; enter NOP
    lda #SFXBYTE_NOP
    jsr enter_data
    jmp main_redraw
++
    cmp #$2f    ; -
    bne ++
    ; enter note off
    lda #$00
    jsr enter_data
    jmp main_redraw
++
    cmp #$d8    ; X
    bne ++
    ; load
    jsr load_but_ask
    jmp main_redraw
++
    cmp #$d7    ; W
    bne ++
    ; save
    jsr save_but_ask
    jmp main_redraw
++
    cmp #$d0    ; P
    bne ++
    ; clear
    jsr clear_but_ask
    jmp main_redraw
++
    jmp main_loop

} ; !zn Main_code


; ###############

!zn Sound_code {
sound_tick:
    lda sound_enabled
    bne +
    rts
+
sound_play_smc_c1 = * + 1
    lda #$00
    sta $900a
    ; * + 1 = sound_play_smc_c1 + 5
    lda #$00
    sta $900b
    lda #$00
    sta $900c
    lda #$00
    sta $900d
    lda #SOUND_VOLUME
    sta $900e

    ldy sound_pos
    lda tbl_mul5,y
    clc
    adc sfxptr
    sta sound_play_smc_pl
    lda tbl_mul5h,y
    adc sfxptrh
    sta sound_play_smc_ph

    ldy #0
-
sound_play_smc_pl = * + 1
sound_play_smc_ph = * + 2
    lda sounddata,y
    cmp #SFXBYTE_NOP
    beq ++
    cmp #SFXBYTE_STOP
    beq sound_play_stop
    ldx tbl_mul5,y
    sta sound_play_smc_c1,x
++  iny
    cpy #LEN_SFXITEM
    bne -
    inc sound_pos
    beq sound_play_stop
    rts

sound_play_stop:
    jmp sound_stop

sound_play:
    jsr sound_stop
    inc sound_enabled
    rts

sound_stop:
    lda #0
    sta sound_enabled
    sta $900a
    sta $900b
    sta $900c
    sta $900d
    sta sound_play_smc_c1
    sta sound_play_smc_c1 + 5 * 1
    sta sound_play_smc_c1 + 5 * 2
    sta sound_play_smc_c1 + 5 * 3
    sta sound_pos
    lda #SOUND_VOLUME
    sta $900e
    sta sound_play_smc_c1 + 5 * 4
    rts

; x = sound id
sound_select:
    stx sound_num
    jsr sound_stop
    lda tbl_soundptr,x
    sta sfxptr
    lda tbl_soundptrh,x
    sta sfxptrh
    lda hexchars,x
    sta vmatrix + 1
    rts

sounddata_clear_all:
    ldx #$f
-   jsr sounddata_clear_x
    dex
    bpl -
    rts

; x = sound id
sounddata_clear_x:
    txa
    pha
    lda tbl_soundptr,x
    sta zpptr
    lda tbl_soundptrh,x
    sta zpptrh
    ldy #0
    ldx #5
    lda #SFXBYTE_NOP
-   sta (zpptr),y
    iny
    bne -
    inc zpptrh
    dex
    bne -
    pla
    tax
    rts

sounddata_load_all:
    ldx #$f
-   jsr sounddata_load_x
    dex
    bpl -
    rts

; x = sound id
set_filename:
    txa
    pha
    lda hexchars_pet,x
    sta filenamedigit
    ldx #<filename
    ldy #>filename
    lda #LEN_filename
    jsr k_setnam
    ldy #0
    ldx k_device_num
    lda #1
    jsr k_setlfs
    pla
    tax
    rts

; x = sound id
sounddata_load_x:
    txa
    pha
    lda #'L' | $80
    sta vmatrix
    lda hexchars,x
    ora #$80
    sta vmatrix + 1
    jsr sounddata_clear_x
    jsr set_filename
    lda tbl_soundptrh,x
    tay
    lda tbl_soundptr,x
    tax
    lda #0
    jsr k_load
    bcs +   ; error
--  lda #'S'
    sta vmatrix
    lda vmatrix + 1
    and #$7f
    sta vmatrix + 1
    pla
    tax
    rts
+   ; error
    lda #'?'
    sta vmatrix
    lda #25
    sta irq_downcount
-   lda irq_downcount
    bne -
    beq -- ; bra


sounddata_save_all:
    ldx #$f
-   jsr sounddata_save_x
    dex
    bpl -
    rts

; x = sound id
sounddata_save_x:
    stx sounddata_save_smc_x
    lda #'S' | $80
    sta vmatrix
    lda hexchars,x
    ora #$80
    sta vmatrix + 1

    ; determine sound end
    lda tbl_soundptr,x
    sta zpptr
    lda tbl_soundptrh,x
    sta zpptrh

    ldy #0
    ldx #0  ; len MSB
-   lda (zpptr),y
    cmp #SFXBYTE_STOP
    beq sounddata_save_got_stop
    iny
    bne -
    inc zpptrh
    inx
    cpx #>LEN_SFXDATA
    bne -

    ; no stop
    ; seek backward for first non-NOP
    dec zpptrh
    dex
    ldy #$ff
-   lda (zpptr),y
    cmp #SFXBYTE_NOP
    bne +
    dey
    cpy #$ff
    bne -
    dec zpptrh
    dex
    bpl -

    ; no data, quit
    ldx sounddata_save_smc_x
    jmp sounddata_save_exit

+   ; got last data, add 1 for end
    iny
    bne sounddata_save_got_len
    inc zpptrh
    bne sounddata_save_got_len  ; bra

sounddata_save_got_stop:
    tya
    clc
    adc #5  ; take whole stop row
    bcc +
    inc zpptrh
+   tay

sounddata_save_got_len:
    sty sounddata_save_end_l
    lda zpptrh
    sta sounddata_save_end_h

sounddata_save_smc_x = * + 1
    ldx #0
    lda tbl_soundptrh,x
    sta zpptrh

    ; scratch old file (if any)
    lda hexchars_pet,x
    sta filenamedigit
    ldx #<filename_s
    ldy #>filename_s
    lda #LEN_filename_s
    jsr k_setnam
    ldy #$f
    ldx k_device_num
    lda #$f
    jsr k_setlfs
    jsr k_open
    lda #$0f
    jsr k_close
    ; save new one
    ldx sounddata_save_smc_x
    jsr set_filename
    lda #0
    sta k_status_word
sounddata_save_end_l = * + 1
    ldx #<0
sounddata_save_end_h = * + 1
    ldy #>0
    lda #zpptr  ; has pointer to start
    jsr k_save
    bcs +   ; error
sounddata_save_exit:
    lda #'S'
    sta vmatrix
    lda vmatrix + 1
    and #$7f
    sta vmatrix + 1
    rts
+   ; error
    lda #'?'
    sta vmatrix
    lda #25
    sta irq_downcount
-   lda irq_downcount
    bne -
    beq sounddata_save_exit ; bra

} ; !zn Sound_code

; ###############


Code_end = *

!if Code_end > $2000 {
!error "code over data: ", * - $2000
}
